module Fuelcell
  module Action
    class ArgDefinition
      NO_DEFAULT_ASSIGNED = '__fuelcell_no_default_assigned__'
      attr_reader :name, :type, :default, :required, :banner

      alias_method :required?, :required

      def initialize(text, config = {})
        @name     = text.to_s
        @required = config[:required] == true ? true : false
        @type     = validate_type(config[:type])
        @default  = config.fetch(:default) { NO_DEFAULT_ASSIGNED }
        @banner   = config[:banner] || ''
      end

      def default?
        @default == NO_DEFAULT_ASSIGNED ? false : true
      end

      private

      def validate_type(value)
        case value.to_s.to_sym
        when :string, :text, :"" then :string
        when :number, :numeric   then :numeric
        when :boolean, :bool     then :bool
        when :hash               then :hash
        when :array              then :array
        else
          fail ArgumentError, "invalid type #{value}"
        end
      end
    end
  end
end
