module Fuelcell
  module Action
    class ArgResults
      extend Forwardable
      attr_reader :raw
      def_delegators :@raw, :empty?, :each, :join

      def initialize(array, hash)
        @raw, @map = validate_results(array, hash)
      end


      def [](key)
        return raw[key] if key.is_a?(Integer)
        value(key)
      end

      protected
      attr_reader :map

      def method_missing(method, *args, &_block)
        method = method.to_s
        return map.key?(method.chomp('?')) if method[-1] == '?'
        value(method)
      end

      def value(key)
        return nil unless index?(key)
        raw[index(key)]
      end

      def index?(key)
        map.key?(key)
      end

      def index(key)
        map[key]
      end

      private

      def validate_results(array, hash)
        hash.each do |(key, value)|
          unless value.is_a?(Integer)
            fail ArgumentError, "hash value for #{key} must be an integer"
          end

          unless array.at(value)
            fail ArgumentError,
                  "arg definition #{key} does not point to value in args array"
          end
        end
        [array, hash]
      end
    end
  end
end
