module Fuelcell
  module Action
    # Used by the Command to manage adding args from its dsl. It is also
    # used during arg parsing to find args, or check if any required args
    # have been missed
    class ArgsManager
      attr_reader :args
      extend Forwardable

      def_delegators :@args, :each, :each_with_index, :empty?

      def initialize
        @args = []
      end

      def args?
        !empty?
      end

      def required
        args.select { |arg| arg.required? }
      end
      alias_method :required_args, :required

      def missing(names)
        list = []
        args.select do |arg|
          list << arg if arg.required? && !names.include?(arg.name)
        end
        return [] if list.empty?

        yield list if block_given?
        list
      end

      def add(arg, config = {})
        arg = create(arg, config) if arg.is_a?(String)
        args.each do |item|
          if item.name == arg.name
            fail "can not add arg: duplicate exists with name #{arg.name}"
          end
        end
        args << arg
      end
      alias_method :arg, :add

      def remove(name)
        # looks like an arg definition so lets use it's name
        name = name.name if name.respond_to?(:name)
        args.delete_if {|arg| arg.name == name}
      end

      def find(name)
        target = false
        args.each {|arg| target = arg if arg.name == name }
        target
      end
      alias_method :find_arg, :find
      alias_method :[], :find

      def create(name, config = {})
        ArgDefinition.new(name, config)
      end
    end
  end
end
