module Fuelcell
  module Action
    # Null Object used to indicate a command has not been found
    class NotFound < Command
      def initialize(cmd_args)
        super(validate_cmd_args(cmd_args))

        @callable = command_not_found_action
        @usage    = ''
        @desc     = 'command not found command'
      end

      def usage(_value = nil)
        @usage
      end

      def desc(_value = nil)
        @desc
      end

      def callable(_value = nil)
        @callable
      end

      def command(key, &block)
        cmd = Command.new(key)
        cmd.instance_eval(&block)
        nil
      end

      def opt(_name, _config = {})
      end

      def <<(_cmd)
      end

      private

      def validate_cmd_args(args)
        args = args.is_a?(String) ? [args] : args
        unless args.respond_to?(:join)
          fail ArgumentError, 'cmd_args must implement join'
        end
        args.join(' ')
      end

      def command_not_found_action
        lambda do |_opts, _args, shell|
          shell.error "command #{name} not found"
          return 1
        end
      end
    end
  end
end
