module Fuelcell
  module Action
    class OptResults < ::Hash

      def initialize(hash = {})
        super()
        hash.each do |key, value|
          self[normalize(key)] = value
        end
      end

      def [](key)
        super(normalize(key))
      end

      def []=(key, value)
        super(normalize(key), value)
      end

      def delete(key)
        super(normalize(key))
      end

      def fetch(key, *args)
        super(normalize(key), *args)
      end

      def key?(key)
        super(normalize(key))
      end

      def values_at(*indices)
        indices.map { |key| self[normalize(key)] }
      end

      def merge(hash)
        dup.merge!(hash)
      end

      def merge!(hash)
        hash.each do |key, value|
          self[normalize(key)] = value
        end
        self
      end

      def to_hash
        Hash.new(default).merge!(self)
      end

      def respond_to?(method_name, include_private = false)
        return key?(method_name.to_s.chomp('?')) || super
      end

      protected

      def normalize(key)
        key.is_a?(Symbol) ? key.to_s : key
      end

      def method_missing(method_name, *args, &_block)
        name = method_name.to_s
        return self.key?(name.chomp('?')) if name[-1] == '?'
        return self[name]
      end
    end
  end
end
