module Fuelcell
  module Action
    module Subcommands
      extend Forwardable
      delegate [:empty?, :[], :each] => :subcommands

      def add(cmds)
        cmds = [cmds] unless cmds.is_a?(Array)
        cmds.each { |cmd|  subcommands[cmd.name] = cmd }
      end
      alias_method :<<, :add

      def [](name)
        list = name.split(' ')
        deep_find(self, list)
      end

      def exist?(name)
        list   = name.split(' ')
        result = deep_find(self, list)
        result.is_a?(NotFound) ? false : true
      end

      # Finds a subcommand by name.
      #
      # @param names [Array] hierarchical order of commands
      # @return [Fuelcell::Command] of nil when not found
      def search(names)
        deep_find(self, names)
      end

      # Collect global options from the option manager of every
      # command in the Hierarchy
      #
      # @return [Hash]
      def global_options(target)
        collect_global_options(target, self, {})
      end

      protected

      def subcommands
        @subcommands ||= {}
      end

      def deep_find(cmd, names)
        return cmd if names.empty?

        name = names.shift
        unless cmd.subcommands.key?(name)
          names.unshift name
          return NotFound.new(names)
        end
        deep_find(cmd.subcommands[name], names)
      end

      def create_tree(current_cmd, tree)
        while search_key = tree.shift
          unless current_cmd.exist?(search_key)
            current_cmd.command(search_key) {}
          end
          child = current_cmd[search_key]
          create_tree(child, tree)
        end
        child
      end

      def collect_global_options(target, cmd, list)
        return list if target === cmd

        globals = cmd.opts.globals
        list.merge!(globals)

        return list if cmd.empty?

        cmd.each do |_, subcommand|
          collect_global_options(target, subcommand, list)
        end
        list
      end
    end
  end
end
