module Fuelcell
  module Help
    class CmdsFormatter < BaseFormatter
      attr_reader :indent, :desc_space
      def initialize(config = {})
        super
        @indent     = (config[:indent] || 2).to_i
        @desc_space = (config[:banner_space] || 2).to_i
      end

      def call(data)
        return '' if empty?(data)
        str    = "Commands:\n"
        widest = widest_cmd(data[:commands])
        data[:commands].each do |cmd|
          str << line(cmd[:name], widest, cmd[:desc])
        end
        str
      end

      def line(cmd, widest_cmd, desc)
        max          = indent + desc_space + widest_cmd
        desc         = wrap(desc || '', max)
        pad          = create_padding(widest_cmd, cmd)
        indent_str   = ' ' * indent
        column_space = ' ' * desc_space
        "#{indent_str}#{cmd}#{pad}#{column_space}# #{desc}\n"
      end

      def create_padding(widest, cmd)
        ' ' * (widest - cmd.size)
      end

      def empty?(data)
        !data.key?(:commands) || data[:commands].empty?
      end

      def wrap(text, widest_text)
        return text if (widest_text + text.size) < max_width

        pad = ' ' * widest_text
        pattern = /(.{1,#{max_width - widest_text}})(\s+|$)/
        text = text.gsub(pattern, "\\1\n#{pad}# ")
        text.gsub(/# $/, '').strip
      end

      def widest_cmd(commands)
        max = 0
        commands.each do |data|
          size = data[:name].size
          max  = size if size > max
        end
        max
      end
    end
  end
end
