module Fuelcell
  module Help
    class OptsFormatter < BaseFormatter
      attr_reader :indent, :banner_space
      def initialize(config = {})
        super
        @indent       = (config[:indent] || 2).to_i
        @banner_space = (config[:banner_space] || 2).to_i
      end

      def call(data)
        return '' if empty?(data)

        str    = "Options:\n"
        widest = widest_opt(data[:options])
        data[:options].each do |opt|
          opt, banner = opt_data(opt)
          str << line(opt, widest, banner)
        end
        str
      end

      def line(opt, widest_opt, banner_text)
        max          = indent + banner_space + widest_opt
        banner       = wrap(banner_text, max)
        pad          = create_padding(widest_opt, opt)
        indent_str   = ' ' * indent
        column_space = ' ' * banner_space
        "#{indent_str}#{opt}#{pad}#{column_space}#{banner}\n"
      end

      def create_padding(widest, opt)
        ' ' * (widest - opt.size)
      end

      def empty?(data)
        !data.key?(:options) || data[:options].empty?
      end

      def wrap(text, widest_text)
        return text if (widest_text + text.size) < max_width

        pad = ' ' * widest_text
        pattern = /(.{1,#{max_width - widest_text}})(\s+|$)/
        text.gsub(pattern, "\\1\n#{pad}").strip
      end

      def widest_opt(options)
        max = 0
        options.each do |data|
          size = "#{short_opt(data, '  ')} #{long_opt(data)}".size
          max  = size if size > max
        end
        max
      end

      def opt_data(data)
        ["#{short_opt(data, '  ')} #{long_opt(data)}", data[:banner] || '']
      end
    end
  end
end
