module Fuelcell
  module Parser
    # Process options that are flags or use another arg as their value. This
    # handles both short and long options. Flags are assigned a value of true
    # while all other values are casted during value assignment
    class OptHandler < BaseHandler
      # @param cmd [Fuelcell::Command]
      # @param args [Array] raw args from ARGV
      # @param opts [Hash] stores the found opts
      # @return [Boolean]
      def call(cmd, args, opts)
        arg = take_first_arg(args) do |text|
          !(text =~ /^(--\w+(?:-\w+)*|-[a-zA-Z])$/).nil?
        end
        return false unless arg

        opt = find_opt(cmd, arg)
        return true if handled_flag?(opts, opt)

        # We know we are not a flag, so we are expecting a value.
        # We also know there are no other values available so for now
        # this is considered a failure condition
        fail_value_required(opt) if args.empty?

        handle_value(opts, opt, arg, args)
        true
      end

      private

      def handle_value(opts, opt, arg, args)
        value = args.shift
        unless arg?(value)
          if opt.default?
            value = opt.default
          else
            fail_value_required(opt)
          end
        end

        assign_opt_value(opts, opt, value, arg)
      end

      def handled_flag?(opts, opt)
        return false unless opt.flag?
        found_opt_flag(opts, opt)
        true
      end

      def fail_value_required(opt)
        return if opt.callable? || opt.cmd_path?

        cli_opts = opt.cli_labels.join(' or ')
        fail "value not found for #{cli_opts} when value is required"
      end
    end
  end
end
