module Fuelcell
  module Parser
    class OptValueEqualHandler < BaseHandler

      # Handle options both long and short that express their value using an
      # equal sign like --foo=bar or -f=bar
      #
      # @param cmd [Fuelcell::Command]
      # @param args [Array] raw args from ARGV
      # @param opts [Hash] stores the found opts
      # @return [Boolean]
      def call(cmd, args, opts)
        return false unless take_first_arg(args) do |arg|
          !!(arg =~ /^(--\w+(?:-\w+)*|-[a-zA-Z])=(.*)$/)
        end

        name  = $1
        value = $2

        opt = find_opt(cmd, name)
        assign_opt_value(opts, opt, value, name)
        true
      end
    end
  end
end
