module Fuelcell
  class Shell
    extend  Forwardable

    SUCCESS_EXIT_CODE      = 0
    ERROR_EXIT_CODE        = 255
    DEFAULT_TERMINAL_WIDTH = 80

    attr_reader :output_stream, :error_stream, :input_stream, :backtrace
    alias_method :backtrace?, :backtrace
    delegate puts: :output_stream

    def initialize(config = {})
      @output_stream  = config[:output_stream] || $stdout
      @input_stream   = config[:input_stream] || $stdin
      @error_stream   = config[:error_stream] || $stderr
      @backtrace      = config[:backtrace] || false
      @terminal_cols  = config[:terminal_cols]
      @backtrace      = @backtrace == true ? true : false
    end

    def enable_backtrace
      @backtrace = true
    end

    def disable_backtrace
      @backtrace = false
    end

    def error(text)
      error_stream.puts text
    end

    def exception(e)
      message = e.message
      message << "\n" << e.backtrace.join("\n") if backtrace?
      error(message)
      failure_exit
    end

    def terminal_width
      if ENV['FUELCELL_COLUMNS']
        result = ENV['FUELCELL_COLUMNS'].to_i
      elsif @terminal_cols
        result = @terminal_cols.to_i
      else
        result = unix? ? dynamic_width : DEFAULT_TERMINAL_WIDTH
      end

      result < 10 ? DEFAULT_TERMINAL_WIDTH : result
    rescue
      DEFAULT_TERMINAL_WIDTH
    end

    # Determine the width using stty or tput
    #
    # credit goes to the rake project, I took this from the application.rb
    # @return Integer
    def dynamic_width
      @dynamic_width ||= (dynamic_width_stty.nonzero? || dynamic_width_tput)
    end

    # Determines if we are on a unix like system for determining the width
    # of the terminal
    #
    # credit goes to the rake project, I took this from the application.rb
    #
    # @return Boolean
    def unix?
      !(
        RUBY_PLATFORM =~
        /(aix|darwin|linux|(net|free|open)bsd|cygwin|solaris|irix|hpux)/i
      ).nil?
    end

    # credit goes to the rake project, I took this from the application.rb
    #
    # @return Integer
    def dynamic_width_stty
      `stty size 2>/dev/null`.split[1].to_i
    end

    # credit goes to the rake project, I took this from the application.rb
    #
    # @return Integer
    def dynamic_width_tput
      `tput cols 2>/dev/null`.to_i
    end

    def successful_exit
      exit(SUCCESS_EXIT_CODE)
    end

    def failure_exit
      exit(ERROR_EXIT_CODE)
    end

    def exit(code)
      Kernel.exit(code)
    end
  end
end
